#' CLcalc:
#' Calculate customized centiles of cervical length according to the PRB/NICHD standard curve
#'
#' @param data: A data frame with the following columns:
#'
#'     GA: Gestational Age (weeks)
#'
#'     Weight: Maternal Weight (kg) between 34 kg and 195 kg.
#'
#'     Height: Maternal Height (cm) between 124 cm and 195 cm
#'
#'     Parity: Number of previous deliveries (Whole number).
#'
#'     CL: Cervical length (mm)
#'
#' @param saveToFile : Logical value indicating whether or not to export the data to a  comma-separated values (CSV) file.
#' @param filename: Name of the CSV file; Only required if saveToFile = TRUE.
#'
#' @return The input data frame with additional columns for the reference centiles (5, 10, 50, 90 and 95) and the percentile of the cervical lengths.
#' @description   This package provides functionality to calculate the customized percentile
#'  of cervical length.
#' @examples
#' # Generate sample data
#' data=data.frame(GA=c(13,15,17),WEIGHT=c(47.63,70.31,99.34),
#' HEIGHT=c(147.32,160,167.64),PARITY=c(2,0,1),CL=c(50,53,42))
#' # Compute the  centile of the cervical length for the sample data
#'  CLcalc(data)
#'

#' @references
#'
#'
#' @export
CLcalc <- function(data,saveToFile=F,filename="CervicalCentiles.csv")
{
  ano=data;
  ano[] <- lapply(ano, function(x) as.numeric(as.character(x)))
  if (!(all(c("GA", "WEIGHT", "HEIGHT","PARITY","CL")
            %in%
            toupper(colnames(ano)))))
  {
    stop("One or more of the required columns, GA, WEIGHT, HEIGHT, PARITY, and CL may be either missing or have a different name in the input data frame")
  }

  colnames(ano)=tolower(colnames(ano))
  ano$cervicallength=ano$cl

  ano = ano[!is.na(ano$weight) & !is.na(ano$height) & !is.na(ano$parity) &
              !is.na(ano$ga) & !is.na(ano$cervicallength) , ]

  ano = ano[ano$ga <= 40 & ano$ga >= 8, ]
  ano = ano[ano$weight >= 34 & ano$weight <= 195, ]
  ano = ano[ano$height >= 124 & ano$height <= 195, ]

  ano$ga=round(ano$ga,1)

  if(nrow(ano)<1)
  {
    stop("Please check the input data frame to ensure that GA, WEIGHT, and HEIGHT are within the acceptable range")
  }

  ano$t1 = (ano$ga - 28)/4
  ano$t2 = ano$t1^2
  ano$t3 = ano$t1^3
  ano$sweight= (ano$weight-77)/22.5
  ano$sweightsquare= (ano$sweight)^2
  ano$sheight= (ano$height-163)/7.2
  ano$dparity1=ifelse(ano$parity==1,1,0)
  ano$dparity2=ifelse(ano$parity>=2,1,0)


  ano$sweightt1=ano$sweight*ano$t1

  ano$sweightt2=ano$sweight*ano$t2
  ano$sweightt3=ano$sweight*ano$t3
  ano$sweightsquaret1=ano$sweightsquare*ano$t1
  ano$sheightt1=ano$sheight*ano$t1
  ano$dparity1t1=ano$dparity1*ano$t1
  ano$dparity2t1=ano$dparity2*ano$t1




  coefs= c(25.592,
           -1.56,
           -0.474,
           -0.033,
           1.671,
           -0.16,
           -0.307,
           1.631,
           2.886,
           0.362,
           -0.061,
           -0.011,
           -0.061,
           -0.125,
           0.005,
           0.172)

  names(coefs)= c("Intercept",
                  "t1",
                  "t2",
                  "t3",
                  "sweight",
                  "sweightsquare",
                  "sheight",
                  "dparity1",
                  "dparity2",
                  "sweightt1",
                  "sweightt2",
                  "sweightt3",
                  "sweightsquaret1",
                  "sheightt1",
                  "dparity1t1",
                  "dparity2t1")



  makeX = function(coefs, ano) {
    for (n in names(coefs)) {
      if (n == "Intercept") {
        X = rep(1, dim(ano)[1])
      }
      if (n %in% names(ano)) {
        X = cbind(X, ano[, n])
      }
    }
    colnames(X)=names(coefs)
    X
  }

  ano2= makeX(coefs,ano)
  rownames(ano2)=rownames(ano)
  pmat=ano2[,names(coefs)]

  qua = pmat %*% coefs


  dat2 = data.frame(cl=ano$cervicallength, y2=(qua),stringsAsFactors = F)
  dat2$stddev=sqrt(69.623+(10.31*ano2[,"t1"])+(1.457*ano2[,"t2"]))

  lambda=1.5; geomn= 35.53676
  invboxcox= function(x){ round((lambda*(geomn^(lambda-1))*x+1)^(1/lambda),1)}

  dat2$cl2= (dat2$cl^lambda-1)/(lambda*geomn^(lambda-1))



  dat2$p5=invboxcox(qnorm(p=0.05, mean = dat2$y2,
                          sd = dat2$stddev, lower.tail = T, log.p = FALSE))
  dat2$p10=invboxcox(qnorm(p=0.1, mean = dat2$y2,
                           sd = dat2$stddev, lower.tail = T, log.p = FALSE))
  dat2$p50=invboxcox(qnorm(p=0.5, mean = dat2$y2,
                           sd = dat2$stddev, lower.tail = T, log.p = FALSE))
  dat2$p90=invboxcox(qnorm(p=0.9, mean = dat2$y2,
                           sd = dat2$stddev, lower.tail = T, log.p = FALSE))
  dat2$p95=invboxcox(qnorm(p=0.95, mean = dat2$y2,
                           sd = dat2$stddev, lower.tail = T, log.p = FALSE))
  dat2$quantile= round(100*pnorm(dat2$cl2,mean = dat2$y2,sd=dat2$stddev) ,1)

  data$p5="";data$p10="";data$p50="";data$p90="";data$p95="";data$Percentile=""
  data[rownames(dat2),paste0("p",c(5,10,50,90,95))]=dat2[,paste0("p",c(5,10,50,90,95))]
  data[rownames(dat2),"Percentile"]=dat2[,"quantile"]
  data[c("p5","p10","p50","p90","p95","Percentile")]=
    lapply( data[c("p5","p10","p50","p90","p95","Percentile")],function(x){as.numeric(as.character(x))})

  if (saveToFile == TRUE) {
    write.csv(data, file = filename)
  }
return(data)
}
